@@ -14,6 +14,7 @@ class StrexTransaction(Model):
             'deliveryMode',
             'statusCode',
             'detailedStatusCode',
+            'statusDescription',
             'smscTransactionId',
             'created',
             'lastModified',
@@ -21,6 +22,7 @@ class StrexTransaction(Model):
             'serviceCode',
             'businessModel',
             'preAuthServiceId',
+            'preAuthServiceDescription',
             'age',
             'isRestricted',
             'smsConfirmation',
